#!/usr/local/bin/ruby

load '~/Sites/_rubylibs/springz.rb'
require '~/Sites/_rubylibs/basiclibrary'
require 'rexml/document'


module GRUSN
	class Season
		@@all=[]
		def self.all
			@@all
		end

		attr_reader :season, :year
		def initialize(season, year)
			@season=season
			@year=year
			@@all << self
		end
	end

	#--------------------------------------------------------------------

	class Team
		@@all=[]
		def self.all
			@@all
		end
		
		
		attr_reader :season, :id, :players, :captains
		def initialize(season,id)
			@season=season
			@id=id
			@captains=[]
			@players=[]
			@@all << self
		end
		
		def add_player(p)
			@players << p
			@captains << p if p.teams_captained.include? self
		end
	end

	#--------------------------------------------------------------------
	
	class Field
		attr_accessor :width, :height, :players
		
		def initialize(width,height)
			@width=width
			@height=height
			@players=[]
		end
		
		def randomize_players
			@players.each{ |p|
				p.move_to( rand(@width), rand(@height) )
			}
		end
		
		def to_svg
			puts '<g id="field"><g id="springz">'
			Springz.all.each{ |s| s.to_svg if s.visible && s.strength>0 }
			puts '</g><g id="players">'
			@players.each{ |p| p.to_svg(:body,false) }
			puts '</g><g id="names">'
			@players.each{ |p| p.to_svg(:name,false) }
			puts '</g></g>'

			@players.each{ |p|
				p.to_svg(:body)
				p.to_svg(:name)
			}
			Springz.all.each{ |s| s.to_svg(true) if s.visible && s.strength>0 }
			
		end
	end

	#--------------------------------------------------------------------

	class Player
		DRAW_SIZE=12
	
		FRAME_DURATION=10
	
		@@lookup={}
		@@all=[]
		def self.all
			@@all
		end


	
		def self.by_name(name)
			@@lookup[name] || self.new(name)
		end

		def self.record_frame
			@@all.each{ |p| p.record_frame }
		end
	
		attr_reader :name, :teams, :teams_captained, :id, :locs

		include Springz::ObjectInterface
		def initialize(name)
			super()
			@name=name
			@size=@springz_radius=DRAW_SIZE
			@teams=[]
			@teams_captained=[]
			@springz_by_player = {}
			@locs=[]
			@@lookup[name] = self
			@id = @@all.length
			@@all << self
		end

		def update_size
			s = DRAW_SIZE + 3*@teams.length
			@size = @springz_radius = s
		end

		def join_with(p2, team=nil)
			spring = Springz.find(self,p2)
			foundSpring = spring
			spring = Springz.new(self,p2,1) if !spring
			joins = @springz_by_player[p2] ||= []
			if team then
				joins.delete(team)
				joins << team
				spring.strength = joins.length*joins.length
				spring.distance = 250/joins.length
			else
				if !foundSpring
					spring.strength=0.05
					spring.distance=700
					spring.hide
				end
			end
		end
		
		def played_on(team)
			@teams << team unless @teams.include? team
			update_size
		end
		def captained(team)
			played_on(team)
			@teams_captained << team unless @teams_captained.include? team
		end
		
		def to_svg(which,anim=true,id=nil)
			last_loc=nil
			case which
				when :body
					x_att = 'cx'
					y_att = 'cy'
					id = 'c' + @id.to_s
					puts %Q|<circle id="#{id}" cx="#{@locs[0].x.round}" cy="#{@locs[0].y.round}" r="#{@size}" />\n| unless anim

				when :name
					x_att = 'x'
					y_att = 'y'
					id = 't' + @id.to_s
					css = @teams_captained.empty? ? '' : %Q| class="cap#{@teams_captained.length}"|
					puts %Q|<text id="#{id}"#{css} x="#{@locs[0].x.round}" y="#{@locs[0].y.round}">#{@name}</text>\n| unless anim
					
				when :o1
					x_att = 'x1'
					y_att = 'y1'

				when :o2
					x_att = 'x2'
					y_att = 'y2'
			end
			if anim
				@locs.each_with_index{ |loc,i|
					if i>0
						x0 = last_loc.x.round
						x1 = loc.x.round
						y0 = last_loc.y.round
						y1 = loc.y.round
						is_last = i==(@locs.length-1)
						#freeze = is_last ? ' fill="freeze"' : ''
						freeze = true ? ' fill="freeze"' : ''
						puts %Q|<animate xlink:href="##{id}" attributeName="#{x_att}" begin="#{(i-1)*FRAME_DURATION+5}s" dur="#{FRAME_DURATION}s" from="#{x0}" to="#{x1}"#{freeze} />\n| if x0!=x1 || is_last 
						puts %Q|<animate xlink:href="##{id}" attributeName="#{y_att}" begin="#{(i-1)*FRAME_DURATION+5}s" dur="#{FRAME_DURATION}s" from="#{y0}" to="#{y1}"#{freeze} />\n| if y0!=y1 || is_last
					end
					last_loc = loc
				}
			end
		end
		
		def record_frame
			@locs << @springz_loc.clone
		end

	end
	
end

#------------------------------------------------------------------------------

class Springz
	def to_svg(anim=false)
		id = 's' + @id.to_s
		if anim
			@o1.to_svg(:o1,anim,id)
			@o2.to_svg(:o2,anim,id)
		else
			puts %Q|<line id="#{id}" class="s#{@strength.to_s.gsub(/\W/,'_')}" x1="#{@o1.locs.first.x.round}" y1="#{@o1.locs.first.y.round}" x2="#{@o2.locs.first.x.round}" y2="#{@o2.locs.first.y.round}" />|
		end
	end
end

#==============================================================================
#------------------------------------------------------------------------------
#==============================================================================


source_xml_file = ARGV[0]

include REXML
player_data = (Document.new(File.new(source_xml_file))).root


t1 = Time.new
player_data.elements.to_a("//season").each{ |xml_season|
	season = GRUSN::Season.new(xml_season.attributes['season'],xml_season.attributes['year'])
	teams = xml_season.elements.to_a('team').each{ |xml_team|
		team = GRUSN::Team.new(season,xml_team.attributes['id'])
		xml_team.elements.to_a('player').each{ |xml_player|
			player = GRUSN::Player.by_name(xml_player.attributes['name'])
			if xml_player.attributes['type']=='captain'
				player.captained(team)
			else
				player.played_on(team)
			end
			team.add_player(player)
		}
	}
}
t2 = Time.new

$stderr.puts "Created #{GRUSN::Player.all.length} players on #{GRUSN::Team.all.length} teams for #{GRUSN::Season.all.length} seasons in #{(t2-t1).round_to(2)} seconds"

t1 = Time.new
GRUSN::Team.all.each{ |team|
	team.players.each_unique_pair{ |p1,p2|
		p1.join_with(p2,team)
		p2.join_with(p1,team)
	}
}
t2 = Time.new

realJoins = Springz.all.length
$stderr.puts "Created #{realJoins} player joins in #{(t2-t1).round_to(2)} seconds"

t1 = Time.new
GRUSN::Player.all.each_unique_pair{ |p1,p2|
	p1.join_with(p2) # no team makes a repulsor join
}
t2 = Time.new

$stderr.puts "Created #{Springz.all.length-realJoins} repulsor joins in #{(t2-t1).round_to(2)} seconds"

Springz.strength_scale = 0.15



field = GRUSN::Field.new(1000,1000)
field.players = GRUSN::Player.all
field.randomize_players


frames = ARGV[1] && ARGV[1].to_i || 5
iterations_per_frame = ARGV[2] && ARGV[2].to_i || 20


frames.times{
	#GRUSN::Player.record_frame
	Springz.apply_all(iterations_per_frame, false)
	2.times{ Springz.prevent_collisions }
}

Springz.apply_all(1, false)
3.times{ Springz.prevent_collisions }


GRUSN::Player.record_frame




puts <<ENDSVG
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.0//EN" "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd" >
<svg xmlns="http://www.w3.org/2000/svg" width="100%" height="100%" viewBox="0 0 #{field.width} #{field.height}" style="overflow:visible" onclick="evt.target.ownerDocument.documentElement.setCurrentTime(0)">
	<style type="text/css">
		#springz line	{ fill:none; stroke:#bdb; stroke-width:1 }
		#springz line.s1	{ stroke-dasharray:1 10 }
		#springz line.s4	{ stroke-width:2 }
		#springz line.s9	{ stroke-width:4; stroke:#9f9 }
		
		#players circle	{ fill:white; stroke-width:1; stroke:#666 }

		#names text	{ text-anchor:middle; font-size:10; baseline-shift:-0.5ex }
		#names text.cap1	{ fill:#600 }
		#names text.cap2	{ fill:#900 }
		#names text.cap3	{ fill:#c00; font-weight:bold }
	</style>
	<rect width="#{field.width}" height="#{field.height}" fill="white" />
ENDSVG
field.to_svg
puts "\n</svg>\n"
